==> ptwordpress-0.0.20-py3-none-any.whl: ptwordpress/ptwordpress.py <==
#!/usr/bin/python3
"""
    Copyright (c) 2025 Penterep Security s.r.o.

    ptwordpress - Wordpress Security Testing Tool

    ptwordpress is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ptwordpress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ptwordpress.  If not, see <https://www.gnu.org/licenses/>.
"""

import argparse
import concurrent.futures
import re
import csv
import os
import sys; sys.path.append(__file__.rsplit("/", 1)[0])
import urllib
import json
import http.client
import requests

from _version import __version__

import threading
import time
from copy import deepcopy
from queue import Queue
from collections import OrderedDict

from ptlibs import ptjsonlib, ptprinthelper, ptmisclib, ptnethelper, ptnethelper
from ptlibs.ptprinthelper import ptprint


from modules.plugins.emails import Emails, get_emails_instance
from modules.plugins.media_download import MediaDownloader
from modules.user_enumeration import UserEnumeration
from modules.source_enumeration import SourceEnumeration
from modules.wpscan_api import WPScanAPI
from modules.routes_walker import APIRoutesWalker
from modules.plugins.hashes import Hashes

from modules.http_client import HttpClient
from modules.helpers import Helpers

from modules.helpers import print_api_is_not_available


class PtWordpress:
    def __init__(self, args):
        self.args                        = args
        self.ptjsonlib: object           = ptjsonlib.PtJsonLib()
        self.base_response: object       = None
        self.rest_response: object       = None
        self.rss_response: object        = None
        self.robots_txt_response: object = None
        self.is_enum_protected: bool     = None # Server returns 429 too many requests error
        self.wp_version: str             = None
        self.http_client                 = HttpClient(args=self.args, ptjsonlib=self.ptjsonlib)
        self.helpers                     = Helpers(args=self.args, ptjsonlib=self.ptjsonlib)
        self.BASE_URL, self.REST_URL     = self.helpers.construct_wp_api_url(args.url)

    def run(self, args) -> None:
        """Main method"""
        self.base_response: object = self._get_base_response(url=self.BASE_URL)
        self.rest_response, self.rss_response, self.robots_txt_response = self.helpers.fetch_responses_in_parallel() # Parallel response retrieval

        self.helpers.check_if_target_is_wordpress(base_response=self.base_response, wp_json_response=None)

        self.is_cloudflare = self.helpers.check_if_behind_cloudflare(base_response=self.base_response)

        self.head_method_allowed: bool      = self.helpers._is_head_method_allowed(url=self.BASE_URL)
        self.target_is_case_sensitive: bool = self.helpers.check_case_sensitivity(url=self.BASE_URL)

        self.SourceFinder: object        = SourceEnumeration(self.BASE_URL, args, self.ptjsonlib, self.head_method_allowed, self.target_is_case_sensitive)
        self.UserEnumerator: object      = UserEnumeration(self.BASE_URL, args, self.ptjsonlib, self.head_method_allowed)
        self.wpscan_api: object          = WPScanAPI(args, self.ptjsonlib)
        self.email_scraper               = get_emails_instance(args=self.args)

        meta_tags = self.helpers.extract_and_print_meta_tags(response=self.base_response)

        self.helpers.parse_site_info_from_rest(rest_response=self.rest_response, base_response=self.base_response, is_cloudflare=self.is_cloudflare)
        Hashes(self.args).get_hashes_from_favicon(response=self.base_response)
        self.helpers.parse_google_identifiers(response=self.base_response)
        self.helpers.extract_and_print_html_comments(response=self.base_response)

        self.wp_version = self.helpers.get_wordpress_version(base_response=self.base_response, rss_response=self.rss_response, meta_tags=meta_tags, head_method_allowed=self.head_method_allowed)

        self.helpers.print_supported_wordpress_versions(wp_version=self.wp_version) # From API
        self.helpers.print_robots_txt(robots_txt_response=self.robots_txt_response)
        self.helpers.process_sitemap(robots_txt_response=self.robots_txt_response)
        self.SourceFinder.discover_xml_rpc()
        self.SourceFinder.wordlist_discovery("admins", title="admin pages", show_responses=True)
        self.SourceFinder.wordlist_discovery("configs", title="configuration files or pages")
        self.SourceFinder.wordlist_discovery("dangerous", title="access to dangerous scripts", method="get")
        self.SourceFinder.wordlist_discovery("settings", title="settings files")
        self.SourceFinder.wordlist_discovery("directories", title="directory listing", search_in_response="index of", method="get")
        self.SourceFinder.wordlist_discovery("fpd", title="Full Path Disclosure vulnerability", method="get")
        self.SourceFinder.wordlist_discovery("logs", title="log files")
        self.SourceFinder.wordlist_discovery("managements", title="management interface")
        self.SourceFinder.wordlist_discovery("informations", title="information pages")
        self.SourceFinder.wordlist_discovery("statistics", title="statistics")
        self.SourceFinder.wordlist_discovery("backups", title="backup files or directories")
        self.SourceFinder.wordlist_discovery("repositories", title="repositories")
        if self.args.read_me:
            self.SourceFinder.wordlist_discovery("readme", title="readme files in root directory")
        else:
            self.SourceFinder.wordlist_discovery("readme_small_root", title="readme files in root directory")

        plugins = self.SourceFinder.run_discovery(response=self.base_response, content_type="plugin")
        themes = self.SourceFinder.run_discovery(response=self.base_response, content_type="theme")

        self.wpscan_api.run(wp_version=self.wp_version, plugins=plugins, themes=themes)
        self.helpers.parse_namespaces_from_rest(rest_response=self.rest_response)

        self.UserEnumerator.run()
        self.email_scraper.print_result()

        media_urls: list = self.SourceFinder.print_media(self.UserEnumerator.get_user_list()) # Scrape all uploaded public media
        if self.args.save_media:
            MediaDownloader(args=self.args).save_media(media_urls)

        self.ptjsonlib.set_status("finished")
        ptprinthelper.ptprint(self.ptjsonlib.get_result_json(), "", self.args.json)

    def _load_url(self, url, args = None, message: str = None):
        try:
            response, dump = ptmisclib.load_url(url, "GET", headers=self.args.headers, cache=self.args.cache, redirects=self.args.redirects, proxies=self.args.proxy, timeout=self.args.timeout, dump_response=True)
            if message:
                ptprint(f"{message}: {response.url}", "TITLE", not self.args.json, colortext=True, end=" ")
                ptprint(f"[{response.status_code}]", "TEXT", not self.args.json, end="\n")
            return response
        except Exception as e:
            if message:
                ptprint(f"{message}: {args.url}", "TITLE", not self.args.json, colortext=True, end=" ")
                ptprint(f"[err]", "TEXT", not self.args.json)
            self.ptjsonlib.end_error(f"Error retrieving response from server.", self.args.json)

    def _handle_redirect(self, response, args):
        if not self.args.json:
            ptprint(f"[{response.status_code}] Returned response redirects to {response.headers.get('location', '?')}, following...", "INFO", not self.args.json, end="", flush=True, newline_above=True)
            ptprint("\n", condition=not self.args.json, end="\n")
            args.redirects = True
            self.BASE_URL = response.headers.get("location")[:-1] if response.headers.get("location").endswith("/") else response.headers.get("location")
            self.BASE_URL = urllib.parse.urlparse(self.BASE_URL)._replace(path='', query='', fragment='').geturl() # Strip path
            self.REST_URL = self.BASE_URL + "/wp-json"
            self.args = args
            self.run(args=self.args)
            sys.exit(0) # Recurse exit.

    def _get_base_response(self, url):
        """Retrieve base response and handle initial redirects"""
        base_response = self._load_url(url=self.BASE_URL, args=self.args, message="Connecting to URL") # example.com/
        self.helpers.print_response_headers(response=base_response)
        if base_response.is_redirect:
            self._handle_redirect(base_response, self.args)
        return base_response


def get_help():
    return [
        {"description": ["Wordpress Security Testing Tool"]},
        {"usage": ["ptwordpress <options>"]},
        {"usage_example": [
            "ptwordpress -u https://www.example.com",
        ]},
        {"Info": [
            "If no wordlist option set, default will be used",
        ]},
        {"options": [
            ["-u",  "--url",                    "<url>",                "Connect to URL"],
            ["-rm",  "--read-me",               "",                     "Enable readme dictionary attacks"],
            ["-o",  "--output",                 "<file>",               "Save emails, users, logins and media urls to files"],
            ["-wpsk", "--wpscan-key",           "<api-key>",            "Set WPScan API key (https://wpscan.com)"],
            ["-sm",  "--save-media",            "<folder>",             "Save media to folder"],
            ["-T",  "--timeout",                "",                     "Set Timeout"],
            ["-p",  "--proxy",                  "<proxy>",              "Set Proxy"],
            ["-c",  "--cookie",                 "<cookie>",             "Set Cookie"],
            ["-a", "--user-agent",              "<agent>",              "Set User-Agent"],
            ["-d", "--delay",                   "<miliseconds>",        "Set delay before each request"],
            ["-ar", "--author-range",           "<author-range>",       "Set custom range for author enumeration (e.g. 1000-1300)"],
            ["-wu", "--wordlist-users",         "<user_wordlist>",      "Set Custom wordlist for user enumeration"],
            ["-H",  "--headers",                "<header:value>",       "Set Header(s)"],
            ["-t",  "--threads",                "<threads>",            "Number of threads (default 10)"],
            ["-r",  "--redirects",              "",                     "Follow redirects (default False)"],
            ["-C",  "--cache",                  "",                     "Cache HTTP communication"],
            ["-v",  "--version",                "",                     "Show script version and exit"],
            ["-h",  "--help",                   "",                     "Show this help message and exit"],
            ["-j",  "--json",                   "",                     "Output in JSON format"],
        ]
        }]

def parse_range(string: str):
    """Parses range, expected formats are 1-999999, 1 9999"""
    match = re.match(r'(\d+)[- ](\d+)$', string)
    try:
        if not match:
            raise argparse.ArgumentTypeError(f"Error: {string} is not in valid format. Expected range in format 1-99999 or 1 99999.")
        if int(match.group(1)) > int(match.group(2)):
            raise argparse.ArgumentTypeError(f"Error: Provided range is not valid")
        if (int(match.group(1)) > 99999) or (int(match.group(2)) > 99999):
            raise argparse.ArgumentTypeError(f"Error: Provided range is too high")

        if int(match.group(1)) < 1:
            return ( 1, int(match.group(2)) )
        return ( int(match.group(1)), int(match.group(2)) )

    except argparse.ArgumentTypeError as e:
        return (1, 10)


def parse_args():
    parser = argparse.ArgumentParser(add_help="False", description=f"{SCRIPTNAME} <options>", allow_abbrev=False)
    parser.add_argument("-u",  "--url",              type=str, required=True)
    parser.add_argument("-p",  "--proxy",            type=str)
    parser.add_argument("-sm",  "--save-media",      type=str)
    parser.add_argument("-wu", "--wordlist-users",   type=str)
    parser.add_argument("-wpsk", "--wpscan-key",     type=str)
    parser.add_argument("-T",  "--timeout",          type=int, default=10)
    parser.add_argument("-t",  "--threads",          type=int, default=10)
    parser.add_argument("-a",  "--user-agent",       type=str, default="Penterep Tools")
    parser.add_argument("-c",  "--cookie",           type=str)
    parser.add_argument("-o",  "--output",           type=str)
    parser.add_argument("-ar", "--author-range",     type=parse_range, default=(1, 10))
    parser.add_argument("-ir", "--id-range",         type=parse_range, default=(1, 10))
    parser.add_argument("-H",  "--headers",          type=ptmisclib.pairs, nargs="+")
    parser.add_argument("-r",  "--redirects",        action="store_true")
    parser.add_argument("-rm",  "--read-me",         action="store_true")
    parser.add_argument("-C",  "--cache",            action="store_true")
    parser.add_argument("-j",  "--json",             action="store_true")
    parser.add_argument("-v",  "--version",          action='version', version=f'{SCRIPTNAME} {__version__}')
    parser.add_argument("--delay",                   type=float, default=0, help="Delay between requests in seconds")
    parser.add_argument("--socket-address",          type=str, default=None)
    parser.add_argument("--socket-port",             type=str, default=None)
    parser.add_argument("--process-ident",           type=str, default=None)
    if len(sys.argv) == 1 or "-h" in sys.argv or "--help" in sys.argv:
        ptprinthelper.help_print(get_help(), SCRIPTNAME, __version__)
        sys.exit(0)

    args = parser.parse_args()

    args.timeout = args.timeout if not args.proxy else None
    args.proxy = {"http": args.proxy, "https": args.proxy} if args.proxy else None
    args.headers = ptnethelper.get_request_headers(args)
    args.threads = 1 if args.delay != 0 else args.threads # Run in one thread if delay parameter provided.
    if args.output:
        args.output = os.path.abspath(args.output)

    ptprinthelper.print_banner(SCRIPTNAME, __version__, args.json, space=0)
    return args

def main():
    global SCRIPTNAME
    SCRIPTNAME = "ptwordpress"
    requests.packages.urllib3.disable_warnings()
    args = parse_args()
    script = PtWordpress(args)
    script.run(args)


if __name__ == "__main__":
    main()

==> ptwordpress-0.0.21-py3-none-any.whl: ptwordpress/ptwordpress.py <==
#!/usr/bin/python3
"""
    Copyright (c) 2025 Penterep Security s.r.o.

    ptwordpress - Wordpress Security Testing Tool

    ptwordpress is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ptwordpress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ptwordpress.  If not, see <https://www.gnu.org/licenses/>.
"""

import argparse
import concurrent.futures
import re
import csv
import os
import sys; sys.path.append(__file__.rsplit("/", 1)[0])
import urllib
import json
import http.client
import requests

from _version import __version__

import threading
import time
from copy import deepcopy
from queue import Queue
from collections import OrderedDict

from ptlibs import ptjsonlib, ptprinthelper, ptmisclib, ptnethelper, ptnethelper
from ptlibs.ptprinthelper import ptprint

from modules.plugins.emails import Emails, get_emails_instance
from modules.plugins.media_download import MediaDownloader
from modules.user_discovery import UserDiscovery
from modules.source_discovery import SourceDiscovery
from modules.wpscan_api import WPScanAPI
from modules.routes_walker import APIRoutesWalker
from modules.plugins.hashes import Hashes

from modules.http_client import HttpClient

from modules.helpers import print_api_is_not_available
from modules.helpers import Helpers


class PtWordpress:
    def __init__(self, args):
        self.args                        = args
        self.ptjsonlib: object           = ptjsonlib.PtJsonLib()
        self.base_response: object       = None
        self.rest_response: object       = None
        self.rss_response: object        = None
        self.robots_txt_response: object = None
        self.is_enum_protected: bool     = None # Server returns 429 too many requests error
        self.wp_version: str             = None
        self.http_client                 = HttpClient(args=self.args, ptjsonlib=self.ptjsonlib)
        self.helpers                     = Helpers(args=self.args, ptjsonlib=self.ptjsonlib)
        self.BASE_URL, self.REST_URL     = self.helpers.construct_wp_api_url(args.url)

    def run(self, args) -> None:
        """Main method"""
        self.base_response: object = self._get_base_response(url=self.BASE_URL)
        self.helpers.BASE_URL = self.BASE_URL
        self.helpers.REST_URL = self.REST_URL

        self.rest_response, self.rss_response, self.robots_txt_response = self.helpers.fetch_responses_in_parallel() # Parallel response retrieval

        self.helpers.check_if_target_is_wordpress(base_response=self.base_response, wp_json_response=None)

        self.is_cloudflare = self.helpers.check_if_behind_cloudflare(base_response=self.base_response)

        self.head_method_allowed: bool      = self.helpers._is_head_method_allowed(url=self.BASE_URL)
        self.target_is_case_sensitive: bool = self.helpers.check_case_sensitivity(url=self.BASE_URL)

        self.SourceDiscovery: object     = SourceDiscovery(self.BASE_URL, args, self.ptjsonlib, self.head_method_allowed, self.target_is_case_sensitive)
        self.UserDiscovery: object       = UserDiscovery(self.BASE_URL, args, self.ptjsonlib, self.head_method_allowed)
        self.wpscan_api: object          = WPScanAPI(args, self.ptjsonlib)
        self.email_scraper               = get_emails_instance(args=self.args)

        meta_tags = self.helpers.extract_and_print_meta_tags(response=self.base_response)

        self.helpers.parse_site_info_from_rest(rest_response=self.rest_response, base_response=self.base_response, is_cloudflare=self.is_cloudflare)
        Hashes(self.args).get_hashes_from_favicon(response=self.base_response)
        self.helpers.parse_google_identifiers(response=self.base_response)
        self.helpers.extract_and_print_html_comments(response=self.base_response)

        self.wp_version = self.helpers.get_wordpress_version(base_response=self.base_response, rss_response=self.rss_response, meta_tags=meta_tags, head_method_allowed=self.head_method_allowed)

        self.helpers.print_supported_wordpress_versions(wp_version=self.wp_version) # From API
        self.helpers.print_robots_txt(robots_txt_response=self.robots_txt_response)
        self.helpers.process_sitemap(robots_txt_response=self.robots_txt_response)
        self.SourceDiscovery.discover_xml_rpc()
        self.SourceDiscovery.wordlist_discovery("admins", title="admin pages", show_responses=True)
        self.SourceDiscovery.wordlist_discovery("configs", title="configuration files or pages")
        self.SourceDiscovery.wordlist_discovery("dangerous", title="access to dangerous scripts", method="get")
        self.SourceDiscovery.wordlist_discovery("settings", title="settings files")
        self.SourceDiscovery.wordlist_discovery("directories", title="directory listing", search_in_response="index of", method="get")
        self.SourceDiscovery.wordlist_discovery("fpd", title="Full Path Disclosure vulnerability", method="get")
        self.SourceDiscovery.wordlist_discovery("logs", title="log files")
        self.SourceDiscovery.wordlist_discovery("managements", title="management interface")
        self.SourceDiscovery.wordlist_discovery("informations", title="information pages")
        self.SourceDiscovery.wordlist_discovery("statistics", title="statistics")
        self.SourceDiscovery.wordlist_discovery("backups", title="backup files or directories")
        self.SourceDiscovery.wordlist_discovery("repositories", title="repositories")
        if self.args.readme:
            self.SourceDiscovery.wordlist_discovery("readme", title="readme files in root directory")
        else:
            self.SourceDiscovery.wordlist_discovery("readme_small_root", title="readme files in root directory")

        plugins = self.SourceDiscovery.run_discovery(response=self.base_response, content_type="plugin")
        themes = self.SourceDiscovery.run_discovery(response=self.base_response, content_type="theme")

        self.wpscan_api.run(wp_version=self.wp_version, plugins=plugins, themes=themes)
        self.helpers.parse_namespaces_from_rest(rest_response=self.rest_response)

        self.UserDiscovery.run()
        self.email_scraper.print_result()

        media_urls: list = self.SourceDiscovery.print_media(self.UserDiscovery.get_user_list()) # Scrape all uploaded public media
        if self.args.save_media:
            MediaDownloader(args=self.args).save_media(media_urls)

        self.ptjsonlib.set_status("finished")
        ptprinthelper.ptprint(self.ptjsonlib.get_result_json(), "", self.args.json)

    def _load_url(self, url, args = None, message: str = None):
        try:
            response, dump = ptmisclib.load_url(url, "GET", headers=self.args.headers, cache=self.args.cache, redirects=self.args.redirects, proxies=self.args.proxy, timeout=self.args.timeout, dump_response=True)
            if message:
                ptprint(f"{message}: {response.url}", "TITLE", not self.args.json, colortext=True, end=" ")
                ptprint(f"[{response.status_code}]", "TEXT", not self.args.json, end="\n")
            return response
        except Exception as e:
            if message:
                ptprint(f"{message}: {args.url}", "TITLE", not self.args.json, colortext=True, end=" ")
                ptprint(f"[err]", "TEXT", not self.args.json)
            self.ptjsonlib.end_error(f"Error retrieving response from server.", self.args.json)

    def _handle_redirect(self, response, args):
        if not self.args.json:
            ptprint(f"[{response.status_code}] Returned response redirects to {response.headers.get('location', '?')}, following...", "INFO", not self.args.json, end="", flush=True, newline_above=True)
            ptprint("\n", condition=not self.args.json, end="\n")
            args.redirects = True
            self.BASE_URL = response.headers.get("location")[:-1] if response.headers.get("location").endswith("/") else response.headers.get("location")
            self.BASE_URL = urllib.parse.urlparse(self.BASE_URL)._replace(path='', query='', fragment='').geturl() # Strip path
            self.REST_URL = self.BASE_URL + "/wp-json"
            self.args = args
            self.run(args=self.args)
            sys.exit(0) # Recurse exit.

    def _get_base_response(self, url):
        """Retrieve base response and handle initial redirects"""
        base_response = self._load_url(url=self.BASE_URL, args=self.args, message="Connecting to URL") # example.com/
        self.helpers.print_response_headers(response=base_response)
        if base_response.is_redirect:
            self._handle_redirect(base_response, self.args)
        return base_response


def get_help():
    return [
        {"description": ["Wordpress Security Testing Tool"]},
        {"usage": ["ptwordpress <options>"]},
        {"usage_example": [
            "ptwordpress -u https://www.example.com",
            "ptwordpress -u https://www.example.com",
            "ptwordpress -u https://www.example.com -wu ~/mywordlist",
            "ptwordpress -u https://www.example.com -o ./example -sm ./media",
        ]},
        {"Info": [
            "If no wordlist option set, default will be used",
        ]},
        {"options": [
            ["-u",  "--url",                    "<url>",                "Connect to URL"],
            ["-rm",  "--readme",               "",                     "Enable readme dictionary attacks"],
            ["-o",  "--output",                 "<file>",               "Save emails, users, logins and media urls to files"],
            ["-wpsk", "--wpscan-key",           "<api-key>",            "Set WPScan API key (https://wpscan.com)"],
            ["-sm",  "--save-media",            "<folder>",             "Save media to folder"],
            ["-T",  "--timeout",                "<timeout>",                     "Set Timeout"],
            ["-p",  "--proxy",                  "<proxy>",              "Set Proxy"],
            ["-c",  "--cookie",                 "<cookie>",             "Set Cookie"],
            ["-a", "--user-agent",              "<agent>",              "Set User-Agent"],
            ["-d", "--delay",                   "<miliseconds>",        "Set delay before each request"],
            ["-ar", "--author-range",           "<author-range>",       "Set custom range for author enumeration (e.g. 1000-1300)"],
            ["-wu", "--wordlist-users",         "<user_wordlist>",      "Set Custom wordlist for user enumeration"],
            ["-H",  "--headers",                "<header:value>",       "Set Header(s)"],
            ["-t",  "--threads",                "<threads>",            "Number of threads (default 10)"],
            ["-r",  "--redirects",              "",                     "Follow redirects (default False)"],
            ["-C",  "--cache",                  "",                     "Cache HTTP communication"],
            ["-v",  "--version",                "",                     "Show script version and exit"],
            ["-h",  "--help",                   "",                     "Show this help message and exit"],
            ["-j",  "--json",                   "",                     "Output in JSON format"],
        ]
        }]

def parse_range(string: str):
    """Parses range, expected formats are 1-999999, 1 9999"""
    match = re.match(r'(\d+)[- ](\d+)$', string)
    try:
        if not match:
            raise argparse.ArgumentTypeError(f"Error: {string} is not in valid format. Expected range in format 1-99999 or 1 99999.")
        if int(match.group(1)) > int(match.group(2)):
            raise argparse.ArgumentTypeError(f"Error: Provided range is not valid")
        if (int(match.group(1)) > 99999) or (int(match.group(2)) > 99999):
            raise argparse.ArgumentTypeError(f"Error: Provided range is too high")

        if int(match.group(1)) < 1:
            return ( 1, int(match.group(2)) )
        return ( int(match.group(1)), int(match.group(2)) )

    except argparse.ArgumentTypeError as e:
        return (1, 10)


def parse_args():
    parser = argparse.ArgumentParser(add_help="False", description=f"{SCRIPTNAME} <options>", allow_abbrev=False)
    parser.add_argument("-u",  "--url",              type=str, required=True)
    parser.add_argument("-p",  "--proxy",            type=str)
    parser.add_argument("-sm",  "--save-media",      type=str)
    parser.add_argument("-wu", "--wordlist-users",   type=str)
    parser.add_argument("-wpsk", "--wpscan-key",     type=str)
    parser.add_argument("-T",  "--timeout",          type=int, default=10)
    parser.add_argument("-t",  "--threads",          type=int, default=10)
    parser.add_argument("-a",  "--user-agent",       type=str, default="Penterep Tools")
    parser.add_argument("-c",  "--cookie",           type=str)
    parser.add_argument("-o",  "--output",           type=str)
    parser.add_argument("-ar", "--author-range",     type=parse_range, default=(1, 10))
    parser.add_argument("-ir", "--id-range",         type=parse_range, default=(1, 10))
    parser.add_argument("-H",  "--headers",          type=ptmisclib.pairs, nargs="+")
    parser.add_argument("-r",  "--redirects",        action="store_true")
    parser.add_argument("-rm",  "--readme",          action="store_true")
    parser.add_argument("-C",  "--cache",            action="store_true")
    parser.add_argument("-j",  "--json",             action="store_true")
    parser.add_argument("-v",  "--version",          action='version', version=f'{SCRIPTNAME} {__version__}')
    parser.add_argument("-d",  "--delay",            type=float, default=0, help="Delay between requests in seconds")
    parser.add_argument("--socket-address",          type=str, default=None)
    parser.add_argument("--socket-port",             type=str, default=None)
    parser.add_argument("--process-ident",           type=str, default=None)
    if len(sys.argv) == 1 or "-h" in sys.argv or "--help" in sys.argv:
        ptprinthelper.help_print(get_help(), SCRIPTNAME, __version__)
        sys.exit(0)

    args = parser.parse_args()

    args.timeout = args.timeout if not args.proxy else None
    args.proxy = {"http": args.proxy, "https": args.proxy} if args.proxy else None
    args.headers = ptnethelper.get_request_headers(args)
    args.threads = 1 if args.delay != 0 else args.threads # Run in one thread if delay parameter provided.
    if args.output:
        args.output = os.path.abspath(args.output)

    ptprinthelper.print_banner(SCRIPTNAME, __version__, args.json, space=0)
    return args

def main():
    global SCRIPTNAME
    SCRIPTNAME = "ptwordpress"
    requests.packages.urllib3.disable_warnings()
    args = parse_args()
    script = PtWordpress(args)
    script.run(args)


if __name__ == "__main__":
    main()
